@@ -1,7 +1,7 @@
 # read in the example data (generated, not real student data)
 sdf <- readNAEP(system.file("extdata/data", "M36NT2PM.dat", package = "NAEPprimer"))
 
-# table to compare to subsequent tables with subsets:
+# table to compare to subsequent tables with subsets
 edsurveyTable(composite ~ dsex, data=sdf, returnMeans=FALSE, returnSepct=FALSE)
 
 # subset to just males
@@ -11,7 +11,7 @@ edsurveyTable(composite ~ dsex, data=newsdf, returnMeans=FALSE, returnSepct=FALS
 
 # Variable names that are not in the sdf get resolved in the parent frame.
 # Practically, that means that the following two subset 
-# calls sdfM1 and sdfM2 do the same thing:
+# calls sdfM1 and sdfM2 do the same thing
 male_var <- "Male"
 sdfM1 <- subset(x=sdf, subset= dsex == male_var)
 sdfM2 <- subset(x=sdf, subset= dsex == "Male")
@@ -32,10 +32,9 @@ dsex_standin <- as.name("dsex")
 sdfM4 <- subset(x=sdf, subset= dsex_standin == "Male")
 table(getData(data=sdfM4, varnames="dsex"))
 
-
 # Here is an example of how one might want to call
 # subset from within a function or loop.
-# First,define a few variables to use dynamically
+# First, define a few variables to use dynamically
 rhs_vars <- c("dsex", "b017451")
 lvls <- c("Male", "Female")
 
@@ -47,3 +46,32 @@ dsdf <- subset(x=sdf, subset=cond, inside=TRUE)
 
 # check the result
 table(getData(data=dsdf, varnames="dsex"))
+
+# returns data, but uses substantial memory
+\dontrun{
+sdf[1:3, "origwt"]
+head(sdf[c("origwt","m145101")])
+sdf[1:3, c("origwt","m145101")]
+sdf[1:3,]
+head(sdf[,14])
+head(sdf[,10:14])
+
+# subset an edsurvey.data.frame.list
+sdfA <- subset(sdf, scrpsu %in% c(5,45,56))
+sdfB <- subset(sdf, scrpsu %in% c(75,76,78))
+sdfC <- subset(sdf, scrpsu %in% 100:200)
+sdfD <- subset(sdf, scrpsu %in% 201:300)
+
+# construct an edsurvey.data.frame.list from these four datasets
+sdfl <- edsurvey.data.frame.list(list(sdfA, sdfB, sdfC, sdfD),
+                                 labels=c("A locations",
+                                           "B locations",
+                                           "C locations",
+                                           "D locations"))
+
+sdfl2 <- subset(sdfl, dsex=="Male")
+# the number of rows in each element of the sdfl
+nrow(sdfl)
+# the number of rows after subsetting each element to just the Males
+nrow(sdfl2)
+}
